#!/usr/bin/perl -w
# $Id$

use strict;

use Data::Denter;
use Test::More tests => 6;
use POE::Component::Generic;
use Symbol ();


my $generic = bless { package=>'P1' }, "POE::Component::Generic";

########## Test package_load
eval { POE::Component::Generic::Child::package_load( 'P1' ) };
is( $@, '', "Didn't try to load P1.pm" );
$generic->{package} = 't::P4';
eval { POE::Component::Generic::Child::package_load( 't::P4' ) };
is( $@, '', "Didn't fail loading t/P4.pm" ) 
    or die "Failure=$@";
my $functions = [ sort Devel::Symdump->functions( "t::P4" ) ];
is_deeply( $functions, [ qw(t::P4::in4) ], "Loaded t/P4.pm" ) 
          or die "functions=", join ', ', @$functions;

######### Test package_map
$generic->package_map();
is_deeply( $generic->{cmd_map}, { qw(
        new P2
        in3f P3
        in22f P22
        in2f P2
        in1f P1 ) }, "Got the package map right")
            or die "cmd_map=", Denter $generic->{cmd_map};


######### Test object_build
my $obj = POE::Component::Generic::Child::object_build( 't::P4', [] );
is_deeply( $obj, (bless {}, 't::P4'), "Built object" );


######### Test methods
$generic->{methods} = [ qw( new in3f in1f ) ];
$generic->{cmd_map} = {};
$generic->package_map();
is_deeply( $generic->{cmd_map}, { qw(
        new P2
        in3f P3
        in1f P1 ) }, "Got the package map right using {methods}")
            or die "cmd_map=", Denter $generic->{cmd_map};




BEGIN {
#######################################################
package P3;
use strict;
use vars qw( @ISA );

BEGIN { 
    require Exporter;
    @ISA = qw( Exporter ) 
};

sub new { my $p=shift; return bless {@_}, $p }
sub in3f {}
sub _no_see_me {}

#######################################################
package P22;
use strict;

sub new { P3::new( @_ ) }
sub in22f {}
sub in2f {}
sub fiddle_carp {}

#######################################################
package P2;
use strict;
use vars qw( @ISA );
require Exporter;

BEGIN { @ISA = qw( P22 ) };

sub in2f {}
sub new { return shift->SUPER::new( @_ ) }
sub __private {}

#######################################################
package P1;
use strict;
use vars qw( @ISA );
require Exporter;

BEGIN { @ISA = qw( P2 P3 ) };


sub in1f {}

}


__END__

$Log$
